/*!
 * @file DRV8462.h
 */

#ifndef __DRV8462_H__
#define __DRV8462_H__

#include <Arduino.h>
#include <SPI.h>


/*

7.5 Register Maps
Table 7-34 lists the memory-mapped registers for the device. All register addresses not listed in Table 7-34
should be considered as reserved locations and the register contents must not be modified.
Table 7-34. Memory Map
Register 7 6 5 4 3 2 1 0 Type Address
FAULT FAULT SPI_ERROR UVLO CPUV OCP STL TF OL R 0x00
DIAG1 OCP_LS2_B OCP_HS2_B OCP_LS1_B OCP_HS1_B OCP_LS2_A OCP_HS2_A OCP_LS1_A OCP_HS1_A R 0x01
DIAG2 STSL OTW OTS STL_LRN_OK STALL LRN_DONE OL_B OL_A R 0x02
DIAG3 RSVD NHOME CNT_OFLW CNT_UFLW RSVD NPOR RSVD R 0x03
CTRL1 EN_OUT SR IDX_RST TOFF [1:0] DECAY [2:0] RW 0x04
CTRL2 DIR STEP SPI_DIR SPI_STEP MICROSTEP_MODE [3:0] RW 0x05
CTRL3 CLR_FLT LOCK [2:0] TOCP OCP_MODE OTSD_MODE OTW_REP RW 0x06
CTRL4 TBLANK_TIME[1:0] STL_LRN EN_STL STL_REP STL_FRQ STEP_FRQ_TOL[1:0] RW 0x07
CTRL5 STALL_TH [7:0] RW 0x08
CTRL6 RC_RIPPLE[1:0] DIS_SSC TRQ_SCALE STALL_TH[11:8] RW 0x09
CTRL7 TRQ_COUNT [7:0] R 0x0A
CTRL8 RSVD TRQ_COUNT[11:8] R 0x0B
CTRL9 EN_OL OL_MODE OL_T[1:0] STEP_EDGE RES_AUTO[1:0] EN_AUTO RW 0x0C
CTRL10 ISTSL[7:0] RW 0x0D
CTRL11 TRQ_DAC[7:0] RW 0x0E
CTRL12 EN_STSL TSTSL_FALL[3:0] RSVD RW 0x0F
CTRL13 TSTSL_DLY[5:0] VREF_INT_EN RSVD RW 0x10
INDEX1 CUR_A_POS[7:0] R 0x11
INDEX2 CUR_A_SIGN RSVD R 0x12
INDEX3 CUR_B_POS[7:0] R 0x13
INDEX4 CUR_B_SIGN RSVD CUR_A[1:0] R 0x14
INDEX5 CUR_A[9:2] R 0x15
CUSTOM_CTRL1 RSVD EN_CUSTOM RW 0x16
CUSTOM_CTRL2 CUSTOM_CURRENT1[7:0] RW 0x17
CUSTOM_CTRL3 CUSTOM_CURRENT2[7:0] RW 0x18
CUSTOM_CTRL4 CUSTOM_CURRENT3[7:0] RW 0x19
CUSTOM_CTRL5 CUSTOM_CURRENT4[7:0] RW 0x1A
CUSTOM_CTRL6 CUSTOM_CURRENT5[7:0] RW 0x1B
CUSTOM_CTRL7 CUSTOM_CURRENT6[7:0] RW 0x1C
CUSTOM_CTRL8 CUSTOM_CURRENT7[7:0] RW 0x1D
CUSTOM_CTRL9 CUSTOM_CURRENT8[7:0] RW 0x1E
ATQ_CTRL1 ATQ_CNT[7:0] R 0x1F
ATQ_CTRL2 ATQ_CNT[10:8] RSVD ATQ_LRN_CONST1[10:8] RW 0x20
ATQ_CTRL3 ATQ_LRN_CONST1[7:0] RW 0x21
ATQ_CTRL4 ATQ_LRN_MIN_CURRENT[4:0] ATQ_LRN_CONST2[10:8] RW 0x22
ATQ_CTRL5 ATQ_LRN_CONST2[7:0] RW 0x23
ATQ_CTRL6 ATQ_UL[7:0] RW 0x24
ATQ_CTRL7 ATQ_LL[7:0] RW 0x25
ATQ_CTRL8 KP[7:0] RW 0x26
ATQ_CTRL9 RSVD KD[3:0] RW 0x27
ATQ_CTRL10 ATQ_EN LRN_START ATQ_FRZ[2:0] ATQ_AVG[2:0] RW 0x28
ATQ_CTRL11 ATQ_TRQ_MIN[7:0] RW 0x29
ATQ_CTRL12 ATQ_TRQ_MAX[7:0] RW 0x2A
ATQ_CTRL13 ATQ_D_THR[7:0] RW 0x2B
ATQ_CTRL14 RSVD RW 0x2C
ATQ_CTRL15 ATQ_ERROR_TRUNCATE[3:0] ATQ_LRN_STEP[1:0] ATQ_LRN_CYCLE_SELECT[1:0] RW 0x2D
ATQ_CTRL16 ATQ_TRQ_DAC[7:0] R 0x2E
ATQ_CTRL17 RSVD VM_SCALE RSVD RW 0x2F
ATQ_CTRL18 RSVD RW 0x30
SS_CTRL1 SS_SMPL_SEL[1:0] RSVD SS_PWM_FREQ[1:0] RSVD EN_SS RW 0x31
SS_CTRL2 SS_KP[7:0] RW 0x32
SS_CTRL3 SS_KI[7:0] RW 0x33
SS_CTRL4 RSVD SS_KI_DIV_SEL[2:0] RSVD SS_KP_DIV_SEL[2:0] RW 0x34
SS_CTRL5 SS_THR[7:0] RW 0x35
CTRL 14 VM_ADC[4:0] RSVD RW 0x3C

*/


// SPI Protocol
#define SPI_ADDRESS_MASK   0x3F00        // Mask for SPI register address bits
#define SPI_ADDRESS_POS    8             // Position for SPI register address bits
#define SPI_DATA_MASK      0x00FF        // Mask for SPI register data bits
#define SPI_DATA_POS       0             // Position for SPI register data bits
#define SPI_RW_BIT_MASK    0x4000        // Mask for SPI register read write indication bit

// Bit Mask definitions for Registers
// Register 0x00: FAULT Register
#define OL_MASK               (0x01)         // Indicates open load condition
#define OT_MASK               (0x02)         // Logic OR of OTW and OTSD
#define STL_MASK              (0x04)         // Indicates motor stall
#define OCP_MASK              (0x08)         // Indicates over current fault condition
#define CPUV_MASK             (0x10)         // Indicates charge pump under voltage fault condition
#define UVLO_MASK             (0x20)         // Indicates an supply under voltage lockout fault condition
#define SPI_ERROR_MASK        (0x40)         // Indication SPI communication error
#define FAULT_MASK            (0x80)         // Inverse value of the nFAULT pin

// Register 0x01 : DIAG1 Register
#define OCP_HS1_A_MASK        (0x01)        // Indicates over current fault on the high-side FET of AOUT1
#define OCP_LS1_A_MASK        (0x02)        // Indicates over current fault on the low-side FET of AOUT1
#define OCP_HS2_A_MASK        (0x04)        // Indicates over current fault on the high-side FET of AOUT2
#define OCP_LS2_A_MASK        (0x08)        // Indicates over current fault on the low-side FET of AOUT2
#define OCP_HS1_B_MASK        (0x10)        // Indicates over current fault on the high-side FET of BOUT1
#define OCP_LS1_B_MASK        (0x20)        // Indicates over current fault on the low-side FET of BOUT1
#define OCP_HS2_B_MASK        (0x40)        // Indicates over current fault on the high-side FET of BOUT2
#define OCP_LS2_B_MASK        (0x80)        // Indicates over current fault on the low-side FET of BOUT2

// Register 0x02 : DIAG2 Register
#define OL_A_MASK             (0x01)        // Indicates open-load detection on AOUT
#define OL_B_MASK             (0x02)        // Indicates open-load detection on BOUT
#define ATQ_LRN_DONE_MASK     (0x04)        // Indicates auto torque learning was successful
#define STALL_MASK            (0x08)        // Indicates motor stall condition
#define STL_LRN_OK_MASK       (0x10)        // Indicates stall detection learning was successful
#define OTS_MASK              (0x20)        // Indicates over temperature shutdown
#define OTW_MASK              (0x40)        // Indicates over temperature warning
#define STSL_MASK             (0x80)        // Indicates motor standstill

// Register 0x03 : DIAG3 Register
#define RSVD0_MASK            (0x01)        // Reserved bit
#define RSVD1_MASK            (0x02)        // Reserved bit
#define NPOR_MASK             (0x04)        // Indicates a VCC UVLO event
#define SILENTSTEP_ERROR_MASK (0x08)        // Indicates Silent Step operation error
#define ATQ_CNT_UFLW_MASK     (0x10)        // Indicates ATQ_CNT is less than ATQ_LL
#define ATQ_CNT_OFLW_MASK     (0x20)        // Indicates ATQ_CNT is more than ATQ_UL
#define NHOME_MASK            (0x40)        // Indicates indexer is at home position of step table
#define SILENTSTEP_ON_MASK    (0x80)        // Indicates that device is working with silentstep decay mode

// Register 0x04 : CTRL1 Register
#define DECAY_MASK            (0x07)        // Bridge decay setting
#define TOFF_MASK             (0x18)        // Current regulation TOFF setting
#define RSVD2_MASK            (0x20)        // Reserved bit
#define SR_MASK               (0x40)        // Output driver rise and fall time selection
#define EN_OUT_MASK           (0x80)        // Hi-Z outputs bit OR-ed with DRVOFF

// Register 0x05 : CTRL2 Register
#define MICROSTEP_MODE_MASK   (0x0F)        // Microstep setting
#define SPI_STEP_MASK         (0x10)        // Enable SPI step control mode
#define SPI_DIR_MASK          (0x20)        // Enable SPI direction control mode
#define STEP_MASK             (0x40)        // Step control bit if SPI_STEP is enabled
#define DIR_MASK              (0x80)        // Direction control bit if SPI_DIR is enabled

// Register 0x06 : CTRL3 Register
#define TW_REP_MASK           (0x01)        // Report OTW on nFAULT
#define OTSD_MODE_MASK        (0x02)        // OTSD latch fault setting
#define OCP_MODE_MASK         (0x04)        // OCP latch fault setting
#define TOCP_MASK             (0x08)        // OCP deglitch time setting
#define LOCK_MASK             (0x70)        // Lock SPI registers
#define CLR_FLT_MASK          (0x80)        // Clear all fault bits

// Register 0x07 : CTRL4 Register
#define STEP_FREQ_TOL_MASK    (0x03)        // Programs the filter setting on the STEP frequency input.
#define RSVD3_MASK            (0x04)        // Reserved bit
#define STL_REP_MASK          (0x08)        // Report stall detection on nFAULT
#define EN_STL_MASK           (0x10)        // Enable stall detection
#define DIS_STL_MASK          (0xEF)        // Disable stall detection - clears EN_STL bit
#define STL_LRN_MASK          (0x20)        // Learn stall count threshold
#define TBLANK_TIME_MASK      (0xC0)        // Controls the current sense blanking time

// Register 0x08 : CTRL5 Register
#define STALL_TH_MASK         (0xFF)        // Stall Threshold Lower 8-bits

// Register 0x09 : CTRL6 Register
#define STALL_TH_MSB_MASK     (0x0F)        // Stall Threshold Upper 4-bits
#define TRQ_SCALE_MASK        (0x80)        // Torque scaling 0: x1 and 1: x8
#define DIS_SSC_MASK          (0x40)        // Spread-spectrum disable for CP and OSC
#define RC_RIPPLE             (0xC0)        // Controls the current ripple in smart tune ripple control decay mode

// Register 0x0A : CTRL7 Register
#define TRQ_COUNT_MASK        (0xFF)        // Torque Count Lower 8-bits

// Register 0x0B : CTRL8 Register
#define TRQ_COUNT_MASK_MSB    (0x0F)        // Torque Count Upper 4-bits
#define REV_ID_MASK           (0xF0)        // Silicon Revision ID

// Register 0x0C : CTRL9 Register
#define EN_AUTO_MASK           (0x01)        // Automatic Microstepping bit
#define RES_AUTO_MASK          (0b00000110)  // Automatic Microstepping Resolution - default of 1/256

// Register 0x0F : CTRL12 Register
#define EN_STSL_MASK          (0x80)        // Standstill power saving mode bit

// Register 0x10 : CTRL13 Register
#define TSTSL_DLY_MASK        (0b11111100)        // Controls the delay between last STEP pulse and activation of standstill power saving mode. 16ms to 1008ms
#define VREF_INT_EN_MASK      (0x02)        // Use the internal 3.3V reference for current regulation.


// Register ATQ_CTRL1 Auto Torque is just ATQ_CNT[7:0]

// Register ATQ_CTRL2 Auto Torque
#define ATQ_CNT_MASK           (0b11100000)  // ATQ_CNT[10:8]
#define ATQ_LRN_CONST_MASK     (0b00000111)  // ATQ_LRN_CONST[10:8]

// Register ATQ_CTRL3 Auto Torque is just ATQ_LRN_CONST[7:0]

// Register ATQ_CTRL4 Auto Torque
#define ATQ_LRN_MIN_CURRENT_MASK (0b11111000)
#define ATQ_LRN_CONST2_MASK      (0b00000111) //  ATQ_LRN_CONST2[10:8]

// Register ATQ_CTRL5 Auto Torque is just ATQ_LRN_CONST2[7:0]

// Register ATQ_CTRL6 Auto Torque is just ATQ_UL

// Register ATQ_CTRL7 Auto Torque is just ATQ_LL

// Register ATQ_CTRL8 Auto Torque is just ATQ_KP

// Register ATQ_CTRL9 Auto Torque is just ATQ_KD

// Register ATQ_CTRL10 Auto Torque
#define ATQ_EN_MASK           (0b10000000)
#define LRN_START_MASK        (0b01000000)
#define ATQ_FRZ_MASK          (0b00111000)
#define ATQ_AVG_MASK          (0b00000111)

// Register ATQ_CTRL11 Auto Torque is just ATQ_TRQ_MIN

// Register ATQ_CTRL12 Auto Torque is just ATQ_TRQ_MAX

// Register ATQ_CTRL13 Auto Torque is just ATQ_D_THR

// Register ATQ_CTRL14 Auto Torque is RSVD

// Register ATQ_CTRL15 Auto Torque
#define ATQ_ERROR_TRUNCATE_MASK   (0b11110000)
#define ATQ_LRN_STEP_MASK         (0b00001100)
#define ATQ_LRN_CYCLE_SELECT_MASK (0b00000011)

// Register ATQ_CTRL16 Auto Torque is just ATQ_TRQ_DAC

// Register ATQ_CTRL17 Auto Torque
#define ATQ_VM_SCALE_MASK   (0b01000000)

// Register ATQ_CTRL18 Auto Torque is RSVD

/// DRV8xx2 SPI Register Addresses
enum
{
  SPI_FAULT             = 0x00,
  SPI_DIAG1             = 0x01,
  SPI_DIAG2             = 0x02,
  SPI_DIAG3             = 0x03,
  SPI_CTRL1             = 0x04,
  SPI_CTRL2             = 0x05,
  SPI_CTRL3             = 0x06,
  SPI_CTRL4             = 0x07,
  SPI_CTRL5             = 0x08,
  SPI_CTRL6             = 0x09,
  SPI_CTRL7             = 0x0A,
  SPI_CTRL8             = 0x0B,
  SPI_CTRL9             = 0x0C,
  SPI_CTRL10            = 0x0D,
  SPI_CTRL11            = 0x0E,
  SPI_CTRL12            = 0x0F,
  SPI_CTRL13            = 0x10,
  SPI_INDEX1            = 0x11,
  SPI_INDEX2            = 0x12,
  SPI_INDEX3            = 0x13,
  SPI_INDEX4            = 0x14,
  SPI_INDEX5            = 0x15,
  SPI_CUSTOM_CTRL1      = 0x16,
  SPI_CUSTOM_CTRL2      = 0x17,
  SPI_CUSTOM_CTRL3      = 0x18,
  SPI_CUSTOM_CTRL4      = 0x19,
  SPI_CUSTOM_CTRL5      = 0x1A,
  SPI_CUSTOM_CTRL6      = 0x1B,
  SPI_CUSTOM_CTRL7      = 0x1C,
  SPI_CUSTOM_CTRL8      = 0x1D,
  SPI_CUSTOM_CTRL9      = 0x1E,
  SPI_ATQ_CTRL1         = 0x1F,
  SPI_ATQ_CTRL2         = 0x20,
  SPI_ATQ_CTRL3         = 0x21,
  SPI_ATQ_CTRL4         = 0x22,
  SPI_ATQ_CTRL5         = 0x23,
  SPI_ATQ_CTRL6         = 0x24,
  SPI_ATQ_CTRL7         = 0x25,
  SPI_ATQ_CTRL8         = 0x26,
  SPI_ATQ_CTRL9         = 0x27,
  SPI_ATQ_CTRL10        = 0x28,
  SPI_ATQ_CTRL11        = 0x29,
  SPI_ATQ_CTRL12        = 0x2A,
  SPI_ATQ_CTRL13        = 0x2B,
  SPI_ATQ_CTRL14        = 0x2C,
  SPI_ATQ_CTRL15        = 0x2D,
  SPI_ATQ_CTRL16        = 0x2E,
  SPI_ATQ_CTRL17        = 0x2F,
  SPI_ATQ_CTRL18        = 0x30,
  SPI_SILENTSTEP_CTRL1  = 0x31,
  SPI_SILENTSTEP_CTRL2  = 0x32,
  SPI_SILENTSTEP_CTRL3  = 0x33,
  SPI_SILENTSTEP_CTRL4  = 0x34,
  SPI_SILENTSTEP_CTRL5  = 0x35,
  SPI_CTRL14            = 0x3C,
};

/// Possible arguments to setStepMode().
typedef enum
{
  /// Full step with 100% current
  MicroStep1_100 = 0b0000,

  /// Full step with 71% current
  MicroStep1     = 0b0001,

  /// Non-circular 1/2 step
  MicroStep2_NC  = 0b0010,

  /// Circular 1/2 step
  MicroStep2     = 0b0011,

  MicroStep4     = 0b0100,
  MicroStep8     = 0b0101,
  MicroStep16    = 0b0110,
  MicroStep32    = 0b0111,
  MicroStep64    = 0b1000,
  MicroStep128   = 0b1001,
  MicroStep256   = 0b1010,
} DRV8462_microstep_t;

/// Possible arguments to setDecayMode().
typedef enum
{
  SlowDecay = 0b000,
  Mixed30Decay = 0b100,
  Mixed60Decay = 0b101,
  SmartTuneDynamicDecay = 0b110,
  SmartTuneRippleControl = 0b111,

} DRV8462_decay_t;

// Structure variables for the DRV8xx2 registers
struct DRV8xx2_REG
{
  uint8_t fault_status_reg = 0;
  uint8_t diag_status1_reg = 0;
  uint8_t diag_status2_reg = 0;
  uint8_t diag_status3_reg = 0;
  uint8_t ctrl1_reg = 0x0F;
  uint8_t ctrl2_reg = 0x06;
  uint8_t ctrl3_reg = 0x38;
  uint8_t ctrl4_reg = 0x49;
  uint8_t ctrl5_reg = 0x03;
  uint8_t ctrl6_reg = 0x20;
  uint8_t ctrl7_reg = 0xFF;
  uint8_t ctrl8_reg = 0x0F;
  uint8_t ctrl9_reg = 0x10;
  uint8_t ctrl10_reg = 0x80;
  uint8_t ctrl11_reg = 0xFF;
  uint8_t ctrl12_reg = 0x20;
  uint8_t ctrl13_reg = 0x10;
  uint8_t ctrl14_reg = 0x58;
  uint8_t index1_reg = 0x80;
  uint8_t index2_reg = 0x80;
  uint8_t index3_reg = 0x80;
  uint8_t index4_reg = 0x82;
  uint8_t index5_reg = 0xB5;
  uint8_t custom_ctrl1_reg = 0x00;
  uint8_t custom_ctrl2_reg = 0x00;
  uint8_t custom_ctrl3_reg = 0x00;
  uint8_t custom_ctrl4_reg = 0x00;
  uint8_t custom_ctrl5_reg = 0x00;
  uint8_t custom_ctrl6_reg = 0x00;
  uint8_t custom_ctrl7_reg = 0x00;
  uint8_t custom_ctrl8_reg = 0x00;
  uint8_t custom_ctrl9_reg = 0x00;
  uint8_t atq_ctrl1_reg = 0x00;
  uint8_t atq_ctrl2_reg = 0x00;
  uint8_t atq_ctrl3_reg = 0x00;
  uint8_t atq_ctrl4_reg = 0x20;
  uint8_t atq_ctrl5_reg = 0x00;
  uint8_t atq_ctrl6_reg = 0x00;
  uint8_t atq_ctrl7_reg = 0x00;
  uint8_t atq_ctrl8_reg = 0x00;
  uint8_t atq_ctrl9_reg = 0x00;
  uint8_t atq_ctrl10_reg = 0x08;
  uint8_t atq_ctrl11_reg = 0x0A;
  uint8_t atq_ctrl12_reg = 0xFF;
  uint8_t atq_ctrl13_reg = 0x05;
  uint8_t atq_ctrl14_reg = 0x0F;
  uint8_t atq_ctrl15_reg = 0x00;
  uint8_t atq_ctrl16_reg = 0xFF;
  uint8_t atq_ctrl17_reg = 0x00;
  uint8_t atq_ctrl18_reg = 0x00;
  uint8_t silentstep_ctrl1_reg = 0x00;
  uint8_t silentstep_ctrl2_reg = 0x00;
  uint8_t silentstep_ctrl3_reg = 0x00;
  uint8_t silentstep_ctrl4_reg = 0x00;
  uint8_t silentstep_ctrl5_reg = 0xFF;
}DRV8xx2_REG_t;

/*!
 *  @brief  Class that stores state and functions for interacting with
 *          DRV8462
 */


// class DRV8462 {
// public:

//   DRV8462(int8_t SPICLK = 18, int8_t SPIMISO = 19, int8_t SPIMOSI = 23, int8_t SPICS = 5);

//   boolean beginSPI ();
//   void setPins(int8_t enablePin = 2, int8_t nSleepPin = 4, int8_t nFaultPin = 15, int8_t stepPin = 16, int8_t dirPin = 17);

//   void FullInitialize(int8_t enablePin = 2, int8_t nSleepPin = 4, int8_t nFaultPin = 15, int8_t stepPin = 16, int8_t dirPin = 17 );

//   void enableOutputs();
//   void disableOutputs();

//   void setDirectionForward();
//   void setDirectionReverse();
//   void toggleDirection();

//   void wakeup_nSLEEP_HIGH();
//   void asleep_nSLEEP_LOW();

//   void clearFaultPulse();
//   void clearFaultSPI();
//   void clearFaults();

//   void setTorqueDAC(uint8_t newDACValue0to255);
//   void setTorqueDACPercent(uint8_t newDACValue0to100);
//   void setRunCurrentPercent(uint8_t newDACValue0to100);

//   void enableAutoMicrostepping();
//   void disableAutoMicrostepping();
//   void setAutoMicroResolution(uint16_t modeNum = 0);

//   void setOutputSlewRate(uint8_t modeNum);
//   void setTOFF(uint8_t modeNum);

//   void setOCP_MODE_autoRecovery();
//   void setOCP_MODE_latchedFault();

//   void enableStandstillPowerSaving();
//   void disableStandstillPowerSaving();
//   void setHoldingCurrent(uint8_t newHoldingCurrent);
//   void setHoldingCurrentPercent(uint8_t newHoldingCurrentPercent);
//   void setStandstillDelayTime(uint16_t newDelayTime);
//   void setStandstillDelayTimeMS(uint16_t newDelayTimeMilliseconds);

//   void setTBLANKTime_us(uint8_t newDelayTime_us);

//   void setMicrostepMode(DRV8462_microstep_t microstep = MicroStep16);
//   void setMicrostepMode(uint16_t modeNum);

//   void setDecayMode(DRV8462_decay_t decayMode = SmartTuneRippleControl);
  
//   void enableAutoTorqueMode();
//   void disableAutoTorqueMode();
//   void ATQ_StartLearning();
//   uint8_t ATQ_ReadLRN_START();
//   uint8_t ATQ_ReadLRN_DONE();
//   void ATQ_enableVMScaling();
//   void ATQ_disableVMScaling();
//   void ATQ_SetInitialLearnCurrent(uint8_t newValue);
//   void ATQ_SetCurrentStepsForLearn(uint8_t newValue);
//   void ATQ_SetCurrentCyclesForLearn(uint8_t newValue);
//   void ATQ_SetMaxCurrentLimit(uint8_t newValue);
//   void ATQ_SetMaxCurrentLimitAsPercent(uint8_t newValue0to100);
//   void ATQ_SetMinCurrentLimit(uint8_t newValue);
//   void ATQ_SetMinCurrentLimitAsPercent(uint8_t newValue0to100);
//   uint8_t ATQ_ReadATQCountUpperLimit();
//   uint8_t ATQ_ReadATQCountLowerLimit();
//   uint8_t ATQ_ReadMinCurrentLimit();
//   uint8_t ATQ_ReadMaxCurrentLimit();
//   void ATQ_SetATQCountUpperLimit(uint8_t newValue);
//   void ATQ_SetATQCountUpperLimitAsPercent(uint8_t newValue0to100);
//   void ATQ_SetATQCountLowerLimit(uint8_t newValue);
//   void ATQ_SetATQCountLowerLimitAsPercent(uint8_t newValue0to100);
//   void ATQ_SetKP(uint8_t newValue);
//   void ATQ_SetKP_AsPercent(uint8_t newValue0to100);
//   void ATQ_SetKD(uint8_t newValue);
//   void ATQ_SetATQ_FRZ(uint8_t newValue);
//   uint8_t ATQ_ReadTRQ_DAC();
//   uint8_t ATQ_ReadTRQ_DAC_AsPercent();
//   uint16_t ATQ_ReadATQ_CNT();
//   float ATQ_ReadATQ_CNTAsPercent();
//   uint16_t ATQ_ReadATQ_LRN_CONST1();
//   uint16_t ATQ_ReadATQ_LRN_CONST2();
//   void ATQ_SetATQ_LRN_CONST1and2(uint16_t CONST1, uint16_t CONST2);


//   SPIClass *_spi; //!< pointer to SPI object
//   uint8_t readRegister(uint8_t reg);
//   void writeRegister(uint8_t reg, uint8_t value);

// private:
//   uint8_t SPIxfer(uint8_t x);
//   uint8_t read8(uint8_t addr);
//   uint16_t read16(uint8_t addr);
//   void write8(uint8_t addr, uint8_t data);

//   // Pin Assignments
//   int8_t _cs, _clk, _miso, _mosi;
//   int8_t _enable, _nsleep, _nfault, _step, _dir;
// };




#endif
